# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui

from prettyqt import core, gui


QtGui.QRegularExpressionValidator.__bases__ = (gui.Validator,)


class RegularExpressionValidator(QtGui.QRegularExpressionValidator):

    def __repr__(self):
        return f"RegularExpressionValidator(RegularExpression({self.get_regex()!r}))"

    def __getstate__(self):
        return dict(regexp=core.RegularExpression(self.regularExpression()))

    def __setstate__(self, state):
        self.__init__()
        self.setRegularExpression(state["regexp"])

    def set_regex(self, regex: str):
        re = core.RegularExpression(regex)
        self.setRegularExpression(re)

    def get_regex(self) -> str:
        val = self.regularExpression()
        return val.pattern()


if __name__ == "__main__":
    val = RegularExpressionValidator()
    val.set_regex(r"\w\d\d")



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore

from prettyqt import core


QtCore.QFileDevice.__bases__ = (core.IODevice,)


class FileDevice(QtCore.QFileDevice):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui, QtWidgets

from prettyqt import gui, widgets, core
from prettyqt.utils import bidict


WRAP_MODES = bidict(none=QtGui.QTextOption.NoWrap,
                    word=QtGui.QTextOption.WordWrap,
                    manual=QtGui.QTextOption.ManualWrap,
                    anywhere=QtGui.QTextOption.WrapAnywhere,
                    boundary_or_anywhere=QtGui.QTextOption.WrapAtWordBoundaryOrAnywhere)

QtWidgets.QPlainTextEdit.__bases__ = (widgets.AbstractScrollArea,)


class PlainTextEdit(QtWidgets.QPlainTextEdit):

    value_changed = core.Signal()

    def __init__(self, text="", parent=None, read_only=False):
        super().__init__(text, parent)
        self.textChanged.connect(self.value_changed)
        self.set_read_only(read_only)

    def __getstate__(self):
        return dict(text=self.text(),
                    enabled=self.isEnabled(),
                    read_only=self.isReadOnly(),
                    font=gui.Font(self.font()))

    def __setstate__(self, state):
        self.__init__()
        self.set_text(state["text"])
        self.setEnabled(state.get("enabled", True))
        self.setFont(state["font"])
        self.setReadOnly(state["read_only"])

    def __add__(self, other):
        if isinstance(other, str):
            self.append_text(other)
            return self

    def append_text(self, text: str):
        self.appendPlainText(text)

    def set_text(self, text: str):
        self.setPlainText(text)

    def text(self) -> str:
        return self.toPlainText()

    def set_read_only(self, value: bool = True):
        """set test to read only

        Args:
            value: True, for read-only, otherwise False
        """
        self.setReadOnly(value)

    def highlight_current_line(self):
        extra_selections = []

        if not self.isReadOnly():
            selection = widgets.TextEdit.ExtraSelection()
            line_color = gui.Color("yellow").lighter(160)
            selection.format.setBackground(line_color)
            selection.format.setProperty(QtGui.QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extra_selections.append(selection)

        self.setExtraSelections(extra_selections)

    @classmethod
    def get_result_widget(cls, *args, **kwargs):
        widget = cls(*args, **kwargs)
        widget.setReadOnly(True)
        widget.set_font("Consolas")
        return widget

    def set_wrap_mode(self, mode: str):
        """set wrap mode

        Allowed values are "none", "word", "manual", "anywhere", "boundary_or_anywhere"

        Args:
            style: wrap mode to use

        Raises:
            ValueError: mode does not exist
        """
        if mode not in WRAP_MODES:
            raise ValueError(f"invalid wrap mode. Allowed values: {WRAP_MODES.keys()}")
        self.setWordWrapMode(WRAP_MODES[mode])

    def set_value(self, value: str):
        self.setPlainText(value)

    def get_value(self):
        return self.text()


if __name__ == "__main__":
    app = widgets.app()
    widget = PlainTextEdit("This is a test")
    widget.show()
    app.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""

import pytest
from prettyqt.utils import helpers


def test_string_to_num_array():
    assert helpers.string_to_num_array("1, 3, 5") == [1, 3, 5]
    with pytest.raises(ValueError):
        helpers.string_to_num_array("1, 2, f")



# -*- coding: utf-8 -*-

"""custom_models module

contains custom models
"""


from .compositevalidator import CompositeValidator
from .notemptyvalidator import NotEmptyValidator
from .notzerovalidator import NotZeroValidator
from .pathvalidator import PathValidator
from .regexvalidators import FloatListValidator, IntListValidator

__all__ = ["PathValidator",
           "NotEmptyValidator",
           "IntListValidator",
           "FloatListValidator",
           "NotZeroValidator",
           "CompositeValidator"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import custom_validators, widgets
from prettyqt.utils import helpers


class ListInput(widgets.LineEdit):

    def __init__(self, default_value="", typ=int, allow_single=False, parent=None):
        super().__init__(parent=parent)
        self.textChanged.connect(self.set_validation_color)
        self.textChanged.connect(self.value_changed)
        if typ is int:
            val = custom_validators.IntListValidator(allow_single=allow_single)
        elif typ is float:
            val = custom_validators.FloatListValidator(allow_single=allow_single)
        else:
            raise ValueError(f"Invalid type {typ}")
        self.set_validator(val)
        self.set_value(default_value)

    def get_value(self) -> list:
        return helpers.string_to_num_array(self.text())

    def set_value(self, value):
        if isinstance(value, list):
            value = str(value)[1:-1].replace(" ", "")
        self.set_text(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = ListInput()
    widget.show()
    app.exec_()
    print(widget.get_value())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import widgets


class ImageViewer(widgets.Widget):

    def __init__(self, title="", path=None, parent=None):
        super().__init__(parent)
        if title:
            self.title = title
        self.image = None
        if path:
            self.image = widgets.Label.image_from_path(path, parent=self)
        self.show()


if __name__ == "__main__":
    app = widgets.app()
    ex = ImageViewer()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui

from prettyqt import gui


QtGui.QIntValidator.__bases__ = (gui.Validator,)


class IntValidator(QtGui.QIntValidator):

    def __repr__(self):
        return f"IntValidator({self.bottom()}, {self.top()})"

    def __getstate__(self):
        return dict(bottom=self.bottom(), top=self.top())

    def __setstate__(self, state):
        self.__init__(state["bottom"], state["top"])


if __name__ == "__main__":
    val = IntValidator()
    val.setRange(0, 9)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore

from prettyqt import core


QtCore.QAbstractProxyModel.__bases__ = (core.AbstractItemModel,)


class AbstractProxyModel(QtCore.QAbstractProxyModel):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore


class RegularExpression(QtCore.QRegularExpression):

    def __repr__(self):
        return f"RegularExpression({self.pattern()!r})"

    def __reduce__(self):
        return (self.__class__, (self.pattern(),))

    def matches_in_text(self, text):
        result = self.match(text)
        while result.hasMatch():
            length = result.capturedLength(0)
            index = result.capturedStart(0)
            yield index, length
            result = self.match(text, index + length)


if __name__ == "__main__":
    reg = RegularExpression()
    reg.setPattern("[0-9]+ [a-z] [0-9]+")
    for i in reg.matches_in_text("033 a 03444"):
        print(i)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets, gui
from prettyqt.utils import bidict


CORRECTION_MODES = bidict(to_previous=QtWidgets.QSpinBox.CorrectToPreviousValue,
                          to_nearest=QtWidgets.QSpinBox.CorrectToNearestValue)

SYMBOLS = bidict(up_down=QtWidgets.QSpinBox.UpDownArrows,
                 plus_minus=QtWidgets.QSpinBox.PlusMinus,
                 none=QtWidgets.QSpinBox.NoButtons)

STEP_TYPES = bidict(default=QtWidgets.QSpinBox.DefaultStepType,
                    adaptive=QtWidgets.QSpinBox.AdaptiveDecimalStepType)


QtWidgets.QAbstractSpinBox.__bases__ = (widgets.Widget,)


class AbstractSpinBox(QtWidgets.QAbstractSpinBox):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLineEdit(widgets.LineEdit())
        self.setGroupSeparatorShown(True)

    def is_valid(self) -> bool:
        return self.hasAcceptableInput()

    def set_validator(self, validator: gui.Validator):
        self.lineEdit().setValidator(validator)

    def get_button_symbols(self) -> str:
        """returns button symbol type

        possible values are "none", "up_down", "plus_minus"

        Returns:
            button symbol type
        """
        return SYMBOLS.inv[self.buttonSymbols()]

    def set_button_symbols(self, mode: str):
        """sets button symbol type

        possible values are "none", "up_down", "plus_minus"

        Args:
            mode: button symbol type to use

        Raises:
            ValueError: invalid button symbol type
        """
        if mode not in SYMBOLS:
            raise ValueError("Invalid button symbol type.")
        self.setButtonSymbols(SYMBOLS[mode])

    def set_correction_mode(self, mode: str):
        """sets correction mode

        possible values are "to_previous", "to_nearest"

        Args:
            mode: correction mode to use

        Raises:
            ValueError: invalid correction mode
        """
        if mode not in CORRECTION_MODES:
            raise ValueError("Invalid correction mode.")
        self.setCorrectionMode(CORRECTION_MODES[mode])

    def get_correction_mode(self) -> str:
        """returns correction mode

        possible values are "to_previous", "to_nearest"

        Returns:
            correction mode
        """
        return CORRECTION_MODES.inv[self.correctionMode()]

    def set_step_type(self, mode: str):
        """sets step type

        possible values are "default", "adaptive"

        Args:
            mode: step type to use

        Raises:
            ValueError: invalid step type
        """
        if mode not in STEP_TYPES:
            raise ValueError("Invalid step type.")
        self.setStepType(STEP_TYPES[mode])

    def get_step_type(self) -> str:
        """returns step type

        possible values are "default", "adaptive"

        Returns:
            step type
        """
        return STEP_TYPES.inv[self.stepType()]

    def set_special_value(self, value: str):
        self.setSpecialValueText(value)

    def get_value(self) -> int:
        return self.value()

    def set_value(self, value: int):
        self.setValue(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = AbstractSpinBox()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import qtawesome as qta
from qtpy import QtWidgets

from prettyqt import widgets, gui


QtWidgets.QToolBox.__bases__ = (widgets.Frame,)


class ToolBox(QtWidgets.QToolBox):

    def __getitem__(self, index):
        if isinstance(index, int):
            return self.widget(index)
        else:
            return self.findChild(QtWidgets.QWidget, index)

    def __getstate__(self):
        children = list()
        for i, widget in enumerate(self.get_children()):
            dct = dict(widget=widget,
                       icon=gui.Icon(self.itemIcon(i)),
                       text=self.itemText(i),
                       enabled=self.isItemEnabled(i),
                       tooltip=self.itemToolTip(i))
            children.append(dct)
        return dict(items=children,
                    current_index=self.currentIndex())

    def __setstate__(self, state):
        self.__init__()
        for i, item in enumerate(state["items"]):
            self.addItem(item["widget"], item["icon"], item["text"])
            self.setItemEnabled(i, item["enabled"])
            self.setItemToolTip(i, item["tooltip"])
        self.setCurrentIndex(state["current_index"])

    def __iter__(self):
        return iter(self.get_children())

    def get_children(self):
        return [self[i] for i in range(self.count())]

    def add_widget(self, widget, title=None, icon=None):
        if title is None:
            title = widget.id
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.addItem(widget, icon, title)
        else:
            self.addItem(widget, title)


if __name__ == "__main__":
    app = widgets.app()
    w = widgets.Widget()
    w2 = widgets.Widget()
    w2.id = "objectName"
    tb = ToolBox()
    tb.add_widget(w, "title")
    tb.add_widget(w2)
    print(len(tb))
    tb.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets

from prettyqt import widgets, core


class FlowLayout(widgets.Layout):

    def __init__(self, parent=None, margin=None, spacing=-1):
        super().__init__(parent)
        if margin is not None:
            self.set_margin(margin)
        self.set_spacing(spacing)
        self.items = []

    def __getstate__(self):
        return dict(items=self.get_children())

    def __add__(self, other):
        if isinstance(other, (QtWidgets.QWidget, QtWidgets.QLayout)):
            self.add(other)
            return self

    def __setstate__(self, state):
        self.__init__()
        for item in state["items"]:
            self.add(item)

    def __getitem__(self, index):
        item = self.itemAt(index)
        widget = item.widget()
        if widget is None:
            widget = item.layout()
        return widget

    def __iter__(self):
        return iter(self.get_children())

    def __del__(self):
        item = self.takeAt(0)
        while item:
            item = self.takeAt(0)

    def __len__(self):
        """needed for PySide2
        """
        return self.count()

    def get_children(self):
        return [self[i] for i in range(self.count())]

    def addItem(self, item):
        self.items.append(item)

    def count(self) -> int:
        return len(self.items)

    def itemAt(self, index):
        if 0 <= index < len(self.items):
            return self.items[index]

        return None

    def takeAt(self, index):
        if 0 <= index < len(self.items):
            return self.items.pop(index)

        return None

    def expandingDirections(self):
        return QtCore.Qt.Orientations(QtCore.Qt.Orientation(0))

    def hasHeightForWidth(self):
        return True

    def heightForWidth(self, width):
        return self.do_layout(core.Rect(0, 0, width, 0), True)

    def setGeometry(self, rect):
        super().setGeometry(rect)
        self.do_layout(rect, False)

    def sizeHint(self):
        return self.minimumSize()

    def minimumSize(self):
        size = core.Size()

        for item in self.items:
            size = size.expandedTo(item.minimumSize())

        margin_width = 2 * self.contentsMargins().top()
        size += core.Size(margin_width, margin_width)
        return size

    def do_layout(self, rect, test_only):
        x = rect.x()
        y = rect.y()
        line_height = 0

        for item in self.items:
            wid = item.widget()
            pb = widgets.SizePolicy.PushButton
            space = self.spacing()
            space_x = space + wid.style().layoutSpacing(pb, pb, QtCore.Qt.Horizontal)
            next_x = x + item.sizeHint().width() + space_x
            if next_x - space_x > rect.right() and line_height > 0:
                x = rect.x()
                space_y = space + wid.style().layoutSpacing(pb, pb, QtCore.Qt.Vertical)
                y = y + line_height + space_y
                next_x = x + item.sizeHint().width() + space_x
                line_height = 0

            if not test_only:
                item.setGeometry(core.Rect(core.Point(x, y), item.sizeHint()))

            x = next_x
            line_height = max(line_height, item.sizeHint().height())

        return y + line_height - rect.y()


if __name__ == '__main__':
    app = widgets.app()
    widget = widgets.Widget()
    layout = FlowLayout()
    layout += FlowLayout()
    layout += widgets.PushButton("Short")
    layout += widgets.PushButton("Longer")
    layout += widgets.PushButton("Different text")
    layout += widgets.PushButton("More text")
    layout += widgets.PushButton("Even longer button text")
    widget.set_layout(layout)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QBoxLayout.__bases__ = (widgets.Layout,)


class BoxLayout(QtWidgets.QBoxLayout):

    def __init__(self, orientation="horizontal", parent=None, margin=None):
        o = self.TopToBottom if orientation == "vertical" else self.LeftToRight
        super().__init__(o, parent)
        if margin is not None:
            self.set_margin(margin)

    def __getitem__(self, index):
        item = self.itemAt(index)
        widget = item.widget()
        if widget is None:
            widget = item.layout()
        return widget

    def __getstate__(self):
        return dict(items=self.get_children(),
                    direction=int(self.direction()))

    def __setstate__(self, state):
        self.__init__()
        direction = self.Direction(state["direction"])
        self.setDirection(direction)
        for item in state["items"]:
            self.add(item)

    def __iter__(self):
        return iter(self.get_children())

    def __len__(self):
        return self.count()

    def __add__(self, other):
        if isinstance(other, (QtWidgets.QWidget, QtWidgets.QLayout)):
            self.add(other)
            return self

    def add(self, *item):
        for i in item:
            if isinstance(i, QtWidgets.QWidget):
                self.addWidget(i)
            else:
                self.addLayout(i)

    def get_children(self):
        return [self[i] for i in range(self.count())]

    def add_stretch(self, stretch: int = 0):
        self.addStretch(stretch)

    def add_spacing(self, size: int):
        self.addSpacing(size)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    layout = BoxLayout("vertical")
    widget = widgets.Widget()
    widget2 = widgets.RadioButton("Test")
    layout += widget2
    widget.set_layout(layout)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore

from prettyqt.utils import bidict
from prettyqt import core


OPEN_MODES = bidict(not_open=QtCore.QIODevice.NotOpen,
                    read_only=QtCore.QIODevice.ReadOnly,
                    write_only=QtCore.QIODevice.WriteOnly,
                    read_write=QtCore.QIODevice.ReadWrite,
                    append=QtCore.QIODevice.Append,
                    truncate=QtCore.QIODevice.Truncate,
                    text=QtCore.QIODevice.Text,
                    unbuffered=QtCore.QIODevice.Unbuffered,
                    new_only=QtCore.QIODevice.NewOnly,
                    existing_only=QtCore.QIODevice.ExistingOnly)

QtCore.QIODevice.__bases__ = (core.Object,)


class IODevice(QtCore.QIODevice):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui

from prettyqt import core


QtGui.QGuiApplication.__bases__ = (core.CoreApplication,)


class GuiApplication(QtGui.QGuiApplication):
    pass



# -*- coding: utf-8 -*-

"""custom_widgets module

"""

from .image import Image
from .listinput import ListInput
from .optionalwidget import OptionalWidget
from .mappedcheckbox import MappedCheckBox
from .stringornumberwidget import StringOrNumberWidget
from .iconlabel import IconLabel
from .flowlayout import FlowLayout
from .colorchooserbutton import ColorChooserButton
from .filechooserbutton import FileChooserButton
from .fontchooserbutton import FontChooserButton
from .inputandslider import InputAndSlider
from .spanslider import SpanSlider
from .labeledslider import LabeledSlider
from .waitingspinner import WaitingSpinner
from .markdownwidget import MarkdownWindow
from .imageviewer import ImageViewer
from .popupinfo import PopupInfo
from .buttondelegate import ButtonDelegate
from .radiodelegate import RadioDelegate
from .selectionwidget import SelectionWidget
from .codeeditor import CodeEditor

__all__ = ["Image",
           "ListInput",
           "OptionalWidget",
           "MappedCheckBox",
           "StringOrNumberWidget",
           "IconLabel",
           "FlowLayout",
           "ColorChooserButton",
           "FileChooserButton",
           "FontChooserButton",
           "InputAndSlider",
           "SpanSlider",
           "LabeledSlider",
           "WaitingSpinner",
           "PopupInfo",
           "ButtonDelegate",
           "RadioDelegate",
           "SelectionWidget",
           "ImageViewer",
           "MarkdownWindow",
           "CodeEditor"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

from prettyqt import widgets
from prettyqt.utils import bidict

TEXT_DIRECTIONS = bidict(top_to_bottom=QtWidgets.QProgressBar.TopToBottom,
                         bottom_to_top=QtWidgets.QProgressBar.BottomToTop)

ALIGNMENTS = bidict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    top=QtCore.Qt.AlignTop,
                    bottom=QtCore.Qt.AlignBottom)


QtWidgets.QProgressBar.__bases__ = (widgets.Widget,)


class ProgressBar(QtWidgets.QProgressBar):
    """Progress dialog

    wrapper for QtWidgets.QProgressBar
    """

    def __init__(self, text_visible=True, parent=None):
        super().__init__(parent=parent)
        self.setTextVisible(text_visible)

    def set_alignment(self, alignment: str):
        """set the alignment of the layout

        Allowed values are "left", "right", "top", "bottom"

        Args:
            mode: alignment for the layout

        Raises:
            ValueError: alignment does not exist
        """
        if alignment not in ALIGNMENTS:
            raise ValueError(f"{alignment!r} not a valid alignment.")
        self.setAlignment(ALIGNMENTS[alignment])

    def get_alignment(self) -> str:
        """returns current alignment

        Possible values: "left", "right", "top", "bottom"

        Returns:
            alignment
        """
        return ALIGNMENTS.inv[self.alignment()]

    def set_text_direction(self, text_direction: str):
        """set the text direction of the layout

        Allowed values are "top_to_bottom", "bottom_to_top"

        Args:
            mode: text direction for the layout

        Raises:
            ValueError: text direction does not exist
        """
        if text_direction not in TEXT_DIRECTIONS:
            raise ValueError(f"{text_direction!r} not a valid text direction.")
        self.setTextDirection(TEXT_DIRECTIONS[text_direction])

    def get_text_direction(self) -> str:
        """returns current text direction

        Possible values are "top_to_bottom", "bottom_to_top"

        Returns:
            Text direction
        """
        return TEXT_DIRECTIONS.inv[self.textDirection()]

    def set_range(self, start, end):
        self.setRange(start, end)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = ProgressBar()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import gui


class IntListValidator(gui.RegExpValidator):

    def __repr__(self):
        return f"IntListValidator(allow_single={self.allow_single})"

    def __init__(self, allow_single=True, parent=None):
        super().__init__(parent)
        self.allow_single = allow_single
        if allow_single:
            self.set_regex(r"^(\d+)(,\s*\d+)*$")
        else:
            self.set_regex(r"^[0-9][0-9\,]+[0-9]$")


class FloatListValidator(gui.RegExpValidator):

    def __repr__(self):
        return f"FloatListValidator(allow_single={self.allow_single})"

    def __init__(self, allow_single=True, parent=None):
        super().__init__(parent)
        self.allow_single = allow_single
        if allow_single:
            self.set_regex(r"^(\s*-?\d+(\.\d+)?)(\s*,\s*-?\d+(\.\d+)?)*$")
        else:
            self.set_regex(r"^(\s*-?\d+(\.\d+)?)(\s*,\s*-?\d+(\.\d+)?)"
                           r"(\s*,\s*-?\d+(\.\d+)?)*$")



# -*- coding: utf-8 -*-

"""Top-level package for PrettyQt."""

__author__ = """Philipp Temminghoff"""
__email__ = "phil65@kodi.tv"
__version__ = "0.63.0"


__all__ = ["core",
           "gui",
           "widgets",
           "custom_widgets",
           "custom_models",
           "syntaxhighlighters",
           "charts",
           "webenginewidgets"]
